#include "ppilib.c"

#define DW		100
#define DH		100
#define ADCW	100
#define ADCH	300
#define DACW	2000
#define DACH	50
#define Y0		50
#define	Y1		250
#define	Y2		(Y0+ADCH+50)

typedef struct {
	int	x,y,w,h;
	}item;

item  dev[25];			/* 25 devices */
double	dacval = 5000.0;
int	smval, digout, digin, cursor=24;
double adcval[8];


void update(void)
{
int	k, color;
double len;

len = (5000.0 + dacval) / 10000.0 * DACW;
block(dev[0].x, dev[0].y, dev[0].x + DACW, dev[0].y + DACH, BLACK);
block(dev[0].x, dev[0].y, dev[0].x + len, dev[0].y + DACH, LIGHTGREEN);
box(dev[0].x, dev[0].y, dev[0].x + DACW, dev[0].y + DACH, WHITE);

digin = readInputs();
for(k = 0; k < 4; ++k)			/* digital inputs */
	{
	if(digin & (1 << k))
		color = LIGHTGREEN;
	else
		color = BLACK;
	block(dev[k+1].x, dev[k+1].y,
		dev[k+1].x + DW, dev[k+1].y + DH, color);
	box(dev[k+1].x, dev[k+1].y,
		dev[k+1].x + DW, dev[k+1].y + DH, WHITE);
	}

for(k = 0; k < 4; ++k)			/* stepper motor */
	{
	if(smval & (1 << k))
		color = LIGHTGREEN;
	else
		color = BLACK;
	block(dev[k+5].x, dev[k+5].y,
		dev[k+5].x + DW, dev[k+5].y + DH, color);
	box(dev[k+5].x, dev[k+5].y,
		dev[k+5].x + DW, dev[k+5].y + DH, WHITE);
	}


for(k = 0; k < 8; ++k)			/* ADC */
	{
	selectADC(k);
	adcval[k] = zeroTo5000();
	len = (adcval[k]) / 5000.0 * ADCH;
	block(dev[k+9].x, dev[k+9].y,
		dev[k+9].x + ADCW, dev[k+9].y + ADCH, BLACK);
	block(dev[k+9].x, dev[k+9].y,
		dev[k+9].x + ADCW, dev[k+9].y + len, LIGHTGREEN);
	box(dev[k+9].x, dev[k+9].y,
		dev[k+9].x + ADCW, dev[k+9].y + ADCH, WHITE);
	}

for(k = 0; k < 8; ++k) 			/* digital output */
	{
	if(digout & (1 << k))
		color = LIGHTGREEN;
	else
		color = BLACK;
	block(dev[k+17].x, dev[k+17].y,
		dev[k+17].x + DW, dev[k+17].y + DH, color);
	box(dev[k+17].x, dev[k+17].y,
		dev[k+17].x + DW, dev[k+17].y + DH, WHITE);
	}
box(dev[cursor].x, dev[cursor].y, dev[cursor].x + dev[cursor].w,
		dev[cursor].y + dev[cursor].h,RED);
}

void initdisp(void)
{
int k;

dev[0].x = 0; dev[0].y = Y2;	/* DAC */
dev[0].w = DACW;
dev[0].h = DACH;
drawText(800, Y2-10,"DAC Output" );

for(k = 0; k < 4; ++k)		/* digital inputs */
	{
	dev[k+1].x = k*(DW+20) + 1000;
	dev[k+1].y = Y1;
	dev[k+1].w = DW;
	dev[k+1].h = DH;
	}
drawText(1070, Y1-30,"Digital IN    Motor Coils" );

for(k = 0; k < 4; ++k)          /* stepper motor */
	{
	dev[k+5].x = k* (DW+20) + 1500;
	dev[k+5].y = Y1;
	dev[k+5].w = DW;
	dev[k+5].h = DH;
	}

for(k = 0; k < 8; ++k)             /* ADC */
	{
	dev[k+9].x = k* (ADCW+20);
	dev[k+9].y = Y0;
	dev[k+9].w = ADCW;
	dev[k+9].h = ADCH;
	}

for(k = 0; k < 8; ++k)	/* digital outputs */
	{
	dev[k+17].x = k* (DW+20) + 1000;
	dev[k+17].y = Y0;
	dev[k+17].w = DW;
	dev[k+17].h = DH;
	}
drawText(300, Y0-20,"ADC Inputs                   Digital OUT" );
drawText(800,950,"PHOENIX");

drawText(0,600,"-> & <- to select field");
drawText(0,550,"Space bar to Toggle Digital outputs.");
drawText(0,500,"up and down cursor to change DAC");
}

void processkey(void)
{
int key;

key= getch();
if(!key)
key = getch();

	box(dev[cursor].x, dev[cursor].y, dev[cursor].x + dev[cursor].w,
		dev[cursor].y + dev[cursor].h,WHITE);
switch(key)
	{
	case ESC:
	closegraph();
	exit(0);
	break;

	case LEFT:
	if(!cursor) cursor = 24; else --cursor;
	break;

	case RIGHT:
	if(cursor == 24) cursor = 0; else ++cursor;
	break;

	case ' ':
		if( (cursor >= 5) && (cursor < 9) )
				smval ^= (1 << (cursor-5));
		if( (cursor >= 17) && (cursor < 25) )
				digout ^= (1 << (cursor-17));
		writeOutputs(digout);
		if( (cursor >= 9) && (cursor < 17) )
			{
			char ss[20];
			selectADC(cursor-9);
			sprintf(ss,"ADC%d %6.1f", cursor-9, zeroTo5000());
			setfillstyle(SOLID_FILL, BLACK);
			drawBar(100,Y0-10,600,Y0-45);
			drawText(100,Y0-20,ss);
			}
	break;


	case UP:
		if(!cursor)
			{
			dacval += 40;
			if(dacval > 5000)
				dacval = 5000.0;
			setVoltage(dacval);
			}
	break;

	case DOWN:
		if(!cursor)
			{
			dacval -= 40;
			if(dacval < -4999.0) dacval = -4999.0;
			setVoltage(dacval);
			}
	break;
	}
	update();
}

void main()
{
int counter;

initialize();
setWorld(0, 0, 2000, 1000);
initdisp();

update();
while(1)
	{
	if(kbhit())
		processkey();
	++counter;
	if(!(counter % 10000))
		update();
	delay(10);
	}

getch();
getch();
closegraph(); 		/* Return the system to text mode	*/
}


