/* ppilib.c -- Parallel Port for root only 18-Dec-2004 BPA */

#include <stdio.h>
#include <stdlib.h>
#include <sys/io.h>
#include <ctype.h>
#include <math.h>

typedef unsigned char	byte;

#define	DATA	0x378		/* printer port I/O addresses */
#define STAT	0x379
#define CTL	0x37A

#define	ADCLO	0               /* select different functions */
#define	ADCHI	1
#define	ADADR	2
#define	ADSTRT	3
#define	DAC	4
#define	SM	5
#define	DOUT	6
#define	DIN	7

#define ENBIT	8	/* to avoid decoder spikes */
#define ADEOC	128	/* connected to S6 of status byte */
#define FALSE	0
#define TRUE	1


void setDAC(byte dat)	/* input 0 to 255 */
{
outb(dat,DATA);		/* DAC goes from -5 to 5 for 0 to 255 */
outb(DAC, CTL);
outb(ENBIT+DAC, CTL);
outb(DAC, CTL);
outb(0,CTL);
}


byte setVoltage(double val)	/* input from -5000 to 5000 mV */
{
static const double m = 10000.0 / 255;
byte	dat;

if( (val < -5000.0) || (val > 5000.0) ) return FALSE;

dat = (byte) (128.0 + val / m);
setDAC(dat);
return TRUE;
}


byte selectADC(byte chan)
{
if(chan > 7) return FALSE;
outb(chan, DATA);
outb(ADADR, CTL);
outb(ENBIT+ADADR, CTL);
outb(ADADR, CTL);
outb(0, CTL);
return TRUE;
}

byte getRawADC(void)		/* returns  0 to 255. used internally */
{
byte	lo,hi,dat;


outb(ADSTRT, CTL);
outb(ENBIT+ADSTRT, CTL);
outb(ENBIT+ADSTRT, CTL);
outb(ADSTRT, CTL);

while((inb(STAT) & ADEOC)) ;		/* wait for EOC to go LOW */

outb(ADCLO, CTL);
outb(ENBIT+ADCLO, CTL);

lo = (inb(STAT) >> 3) & 15;

outb(ADCLO, CTL);
outb(ADCHI, CTL);
outb(ENBIT+ADCHI, CTL);
hi = (inb(STAT) >> 3) & 15;

dat = (hi << 4) | lo;
//printf("lo = %x hex hi = %x hex ADC data = %x hex\n", lo, hi, dat);
return dat;
}

double zeroTo5000(void)		/* returns  0 to 5000 mV */
{
static const double m = 5000.0 / 255;
return (double) m * getRawADC();
}

double minus5000To5000(void)	/* returns  -5000 to 5000 mV */
{
static const double m = 10000.0 / 255;
return (double) m * getRawADC() - 5000;
}


static int smDelay = 5000;

int setMotorDelay(int k)
{
if( (k < 200) || (k <0 )) return FALSE;
smDelay = k;
return TRUE;
}

void rotateMotor(int nsteps , int dir)
{
static byte pos = 0;
static byte seq[4] = {12, 6, 3 ,9};
int i;

for( i = 0; i < nsteps; ++i)
	{
	if(dir)
	   if(pos == 3) pos = 0; else ++pos;
	else
	   if(pos == 0) pos = 3; else --pos;

	outb(seq[pos], DATA);
	outb(SM, CTL);
	outb(SM+ENBIT, CTL);
	outb(SM, CTL);
	outb(0, CTL);
	usleep(smDelay);
	}
}


byte readInputs(void)
{
byte dat;

outb(DIN, CTL);
outb(ENBIT+DIN, CTL);
dat = inb(STAT) >> 3;
outb(DIN, CTL);
return dat & 15;
}


void writeOutputs(byte dat)
{
outb(dat, DATA);
outb(DOUT, CTL);
outb(ENBIT+ DOUT, CTL);
outb(DOUT, CTL);
}


void writeStepper(byte dat)
{
outb(dat, DATA);
outb(SM, CTL);
outb(ENBIT+ SM, CTL);
outb(SM, CTL);
}


main()
{
int k, v1,v2;


if(ioperm(DATA,3,1) < 0) exit(1);


selectADC(0); 

setDAC(200);
usleep(1000);


for(k=128; k < 256; ++k)
	{
	setDAC(k); usleep(10);
	printf("%d %d\n", k, 128 + getRawADC()/2);
	}
}
