#!/usr/bin/python
"""
A user interface class implemented using pygame. A plot are and a group of buttons placed
below are implemented. Buttons are added along with a callback function that is not a
member of the class.

"""

import pygame, sys, time
from Tkinter import *
from pygame.locals import *
from pygrace import *
import phoenix

# global variables
NP = 100						# Number of points in x direction
width = 500						# diplay window dimensions
height = 400

w = None						# Global Pwin object
p = phoenix.phoenix()			# The Phoenix object
if p == None:
	root = Tk()
	Label(root,text='Error: Phoenix open failed. May be another program using it').pack()
	root.mainloop()
	sys.exit() 
p.set_timeout(10000)

delay = 20000					# Phoenix ADC conversion time


#parameters required by Pwin for scaling the graphs
xmin = 0
xmax = NP
ymin = 0.0						# This will be zero for unipolar ADC inputs
ymax = 2000.0

looping = False					# global variable for programs like CRO
trig = 0.0						# used by CRO
phdata = []						# data from phoenix

class Pwin:
	""" A class to manage a window with a plot area and several buttons"""
	global xmin, xmax, ymin, ymax, NP, phdata
	border = 3
	menuh = 30
	plotw = width - border * 2
	ploth = height - menuh - border * 4
	plotrect = pygame.Rect(border, border, plotw+border, ploth+border)
	xscale = plotw / (xmax - xmin)
	yscale = ploth / (ymax - ymin)
	white = (255, 255, 255)
	black = (0, 0, 0)
	green = (0, 100, 0)
	numButtons = 0
	left = 0;					# static used by addButton()
	buttons = []
	bg = None					# pygame Surface object
	scr = None					# pygame display object
	font = None
	points = []					# Array of points required by pygame.draw_lines
	
	def __init__(self, title, width = 500, height = 400):
		pygame.init()
		self.scr = pygame.display.set_mode((width, height))
		pygame.display.set_caption(title)
		self.bg = pygame.Surface(self.scr.get_size())
		self.bg = self.bg.convert()
		self.bg.fill(self.black)
		self.font = pygame.font.Font(None, 24)
		for x in range(NP):					# Is there a better way to create this array
			self.points.append([0,0])  		# this is made global to avoid creating it every time

	def addButton(self, name, func):
		""" Adds a Button with label 'name' and make the binding to call 'func' when clicked
		"""
		text = self.font.render(name , 1, self.white)
		textrect = text.get_rect()
		textrect.bottomleft = (self.border + self.left, height - self.border)
		pygame.draw.rect(self.bg, self.green, textrect, 0)
		self.bg.blit(text, textrect)
		self.buttons.append((name, func, textrect))	
		self.left = self.left + textrect.width + 2 * self.border
		self.numButtons = self.numButtons + 1

	def refresh(self):
		self.scr.blit(self.bg, (0,0))
		pygame.display.flip()

	def clear_plot(self):
		self.bg.fill(self.black, self.plotrect)
	
	def draw_line(self, x1, y1, x2, y2):
		"""	plots a line from (x1,y1) to (x2,y2). Assume them as (time,voltage) pairs
			Also uses xscale, yscale and ymin to scale the coordinates to plot window
		"""
		x1  = self.border + x1 * self.xscale
		x2  = self.border + x2 * self.xscale
		y1 = self.border + self.ploth - (y1 - ymin) * self.yscale
		y2 = self.border + self.ploth - (y2 - ymin) * self.yscale

#		print x1, ' ', y1, ' ', x2, ' ', y2

		pygame.draw.line(self.bg, self.white, (x1,y1), (x2,y2), 1)
		self.refresh()

	def draw_lines(self):
		"""	plots lines using the global variable points.
			Also uses xscale, yscale and ymin to scale the coordinates to plot window
		"""
		self.clear_plot()
		i = 0
		while i < NP:
			self.points[i][0] = self.border + phdata[i][0] * self.xscale
			self.points[i][1] = self.border + self.ploth - (phdata[i][1] - ymin) * self.yscale
			i = i + 1
		pygame.draw.lines(self.bg, self.white, 0, self.points, 1)
		self.refresh()


	def disp_image(self,img):
		pic = pygame.image.load(img)
		rect = pic.get_rect()
		rect.right = self.plotw		# restrict image to plot area
		rect.bottom = self.ploth
		self.scr.blit(pic, rect)
		pygame.display.flip()

	def	userInput(self):									# Event loop
		""" infinite loop lokking for mouse down events"""
		global looping
		while 1:
			for e in pygame.event.get():
				if e.type == MOUSEBUTTONDOWN:
					x = 0
					cmd = None
					while x < self.numButtons:
						left = self.buttons[x][2][0]
						top = self.buttons[x][2][1]
						right = left + self.buttons[x][2][2]
						bottom = top + self.buttons[x][2][3]
						if left < e.pos[0] < right and top < e.pos[1] < bottom:
							self.buttons[x][1]()		# call the associated function
						x = x +1
			periodic()
#-------------------------------------class end ----------------------------------------			
index = 0
start = 0
pend = []				# Array for distance vs time
for x in range(NP):		# Is there a better way to create this array
	pend.append([0,0])  # this is made global to avoid creating it every time

def	periodic():		
	""" Called continuosly by the event handler loop. Anything that has to be done in a loop
	call be called from here by setting appropriate global flags.
	"""
	global looping
	if looping:
		read_echo()
		
def	read_echo():
	"""
	Reads 2 points from echo
	"""
	global	pend, delay, index,looping, NP, start, ymax
	
	tm1 = p.pulse2ftime(0, 0, 13, 30)
	if(tm1 < 0):
		tm1 = ymax
	pend[index][0] = index
	pend[index][1] = tm1

	time.sleep(0.02)

	if(index > 1):
		w.draw_line(pend[index-1][0], pend[index-1][1], pend[index][0], pend[index][1])

#	print index,' ',pend[index][0], ' ', pend[index][1]	

	index = index + 1
	if index == NP:
		looping = False

def	start():
	global looping, index
	looping = True
	index = 0
	w.clear_plot()

def stop():
	global looping
	looping = False
	
def	help():
	w.disp_image("/usr/phoenix/python/pics/echo.jpg")

def save():
	global t,v, NP
	f = open('echo.dat','w')
	i = 0
	while i < NP:				# data returned by read_block goes to file
		x = pend[i][0]
		y = pend[i][1]
		s = str(x) + ' ' + str(y) + '\n'	
		f.write(s)	
		i = i + 1
	f.close()	

def analyse():
	global NP, pend, looping
	if looping:
		return
	pg = grace()
	x = range(NP)
	y = range(NP)
	i = 0
	while i < NP:				# data returned by read_block goes to xmgrace
		x[i] = pend[i][0]		
		y[i] = pend[i][1]	
		i = i + 1
	pg.plot(x,y)


def	quit():
	print 'bye bye'
	sys.exit()

#p.set_timeout(50000)

w = Pwin("Phoenix Pendulum Program")
w.addButton("Start", start)
w.addButton("Stop", stop)
w.addButton("Save", save)
w.addButton("Grace",analyse)
w.addButton("Help", help)
w.addButton("Quit", quit)

w.refresh()			# display everything now
w.userInput()		# The main loop
