// Library for scilab

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#include "phdriver.h"

#define FALSE		0
#define TRUE		1

/* #define MAXPOINTS	(1 << 13)
   #define MAXDELAY	1000
*/

/*
A collection of functions callable from scilab. All functions return zero only.
The variables are passed back and forth using arguments passed by address
*/

static int handle = -1;

int 
openPhoenix_(void)
{
/* 
For this call to succeed the driver should be loaded earlier using 'insmod' 
command and the device file '/dev/phoenix' should exist. 
*/

  if (handle > 0) return 0;
  handle = open ("/dev/phoenix", O_RDWR);
  if (handle < 0)
      fprintf (stderr, "Phoenix driver open failed\n");
  printf("handle = %d\n", handle);
  return 0;
}


int
readInputs_ (int* dat)
{
/*
Returns a value from 0 to 15 depends on the voltage levels present at the
four digital input pins of the phoenix box. Returns -1 on error
*/
  int tmp;

  if (ioctl (handle, DIGIN, &tmp) < 0)
    {
      fprintf (stderr, "DIGIN failed\n");
      *dat = -1;		// error return
    }
    *dat = tmp & 15;
  return 0;
}


int 
writeOutputs_ (int *dat)
{
/* Writes a value from 0 to 255 to the eight digital outputs of phoenix
*/

  int w = *dat;
  printf("handle = %d data = %d\n", handle, w);

  if (ioctl (handle, DIGOUT, &w) < 0)
      fprintf (stderr, "DIGOUT failed\n");

  return 0;
}

int
setDAC_ (int *dat)		/* input 0 to 255 */
{
/*
Sets the output of the programmable voltage source. It is an 8 bit DAC that
takes 0 to 255 input and sets output from -5V to +5V.
*/

  int dac = *dat;
  printf("handle = %d dac = %d\n", handle, dac);

  if (ioctl (handle, SETDAC, &dac) < 0)
      fprintf (stderr, "SETDAC failed\n");
  return 0;
}

int
selectADC_ (int *chan)
{
  if (*chan > 7) return 0;

  if (ioctl (handle, SELECTADC, chan) < 0)
      fprintf (stderr, "SELECTADC failed\n");

  return 0;
}

int
readBlock_ (int *n, int *d, int *b,  double *t, double *v)
{
  int npoints = *n;
  int delay = *d;
  int bipolar = *b;
  drvdata ddata[MAXPOINTS];
  int k, *ip;
  double strt;

  if (npoints > MAXPOINTS)
    {
    fprintf(stderr, "Number of samples limited to %d\n", MAXPOINTS);
    return 0;
    }
  ip = (int *) ddata;
  *ip++ = npoints;
  *ip = delay;
  if (ioctl (handle, READBLOCK, ddata) < 0)
    {
      fprintf (stderr, "READBLOCK failed\n");
      return 0;
    }

  strt = (double) ddata[0].t.tv_sec + (double) ddata[0].t.tv_usec / 1.0e6;

  for (k = 0; k < npoints; ++k)
    {
      t[k] = (double) ddata[k].t.tv_sec +
            (double) ddata[k].t.tv_usec / 1.0e6 - strt;

      if (bipolar)
	    v[k] = 10000.0 / 255.0 * (ddata[k].adval[0] & 0xff) - 5000;
      else
	    v[k] = 5000.0 / 255.0 * (ddata[k].adval[0] & 0xff);
    }
return 0;
}


