/* Compile  using,
cc -Wall -o mca mca.c -lXm -lXpm -lXext -lXt -lX11 -lm -L/usr/X11R6/lib
To learn the X-Window code try using the manpages like,
# man XtVaAppInitialize
*/

#include "phlib.h"

#include <stdlib.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/DrawingA.h>
#include <Xm/Label.h>

				// defined only to save some typing later
#define	FALSE		0
#define	TRUE		1
#define	Fix_L		XmNleftAttachment, XmATTACH_FORM
#define	Fix_T		XmNtopAttachment, XmATTACH_FORM
#define	Fix_R		XmNrightAttachment, XmATTACH_FORM
#define	Fix_B		XmNbottomAttachment, XmATTACH_FORM
#define	Fix_LTRB	Fix_L, Fix_T, Fix_R, Fix_B
#define	XTIMER		5	// 10 msec

char *prog_name = "Pendulum";
XtAppContext apc;
Widget top, mainForm, plotForm, commandForm;
Widget draw, start, stop, save, tval, quit;
Window dispwin;
GC gc;
Colormap cm;
Display *dpy;
Pixel fgcol, bgcol;

#define	NP	100
double ts[NP], ad[NP], strt;	// arrays to store adc data
int curpos;			// current position

unsigned char running;		// digitization in progress
double starttime, xscale, yscale, m = (10000.0 / 256.0), c = 5000.0;
int height;

void
set_label_string (Widget w, char *p)
{
  XmString s;

  s = XmStringCreateLocalized (p);
  XtVaSetValues (w, XmNlabelString, s, NULL);
  XmStringFree (s);
}

void
timer (void)
{
  int x, y;
  char ss[50];

  if (running)
    {
      ts[curpos] = (double) curpos;
      ad[curpos] = f2rTime (0, 0);

      x = (int) (xscale * curpos);
      y = height - (int) (ad[curpos] * yscale);
//      printf("%d %f %d %d\n", curpos, ad[curpos], x, y);
      XDrawPoint (dpy, dispwin, gc, x, y);

      sprintf (ss, " %7.5lf usec", ad[curpos]);
      set_label_string (tval, ss);

      if (curpos < NP - 1)
	++curpos;
      else
	running = FALSE;
    }

  XtAppAddTimeOut (apc, XTIMER, (XtTimerCallbackProc) timer,
		   (XtPointer) NULL);
}


void
startCollect (Widget wig, XtPointer client_data, XtPointer call_data)
{
  Dimension w, h;

  XClearWindow (dpy, dispwin);
  XSetFunction (dpy, gc, GXcopy);
  XSetForeground (dpy, gc, fgcol);
  XtVaGetValues (draw, XmNwidth, &w, XmNheight, &h, NULL);

  xscale = (double) w / NP;
  yscale = (double) h / 10000.0;	// 10 msec max

  height = (int) h;

//getLowTime(0) * 0.8;  

  curpos = 0;

//printf("w= %d h= %d xs = %f ys = %f\n",w,h,xscale,yscale);

  printf ("Started..\n");
  running = TRUE;
}


void
saveData (Widget w, XtPointer client_data, XtPointer call_data)
{
  int k;
  FILE *fp;

  fp = fopen ("pulse.dat", "wt");
  for (k = 0; k < NP; ++k)
    fprintf (fp, "%8.6f %6.1f\n", ts[k], ad[k]);
  fclose (fp);
  fprintf (stderr, "file saved.\n\a");
}

void
stopCollect (Widget w, XtPointer client_data, XtPointer call_data)
{
  running = FALSE;
}

void
bye (Widget w, XtPointer client_data, XtPointer call_data)
{
  printf ("bye bye\n");
  exit (0);
}


//---------------------------Initialization-------------------------------
void
initGraphics ()			// get all the Drawing Stuff for draw widget
{
  XGCValues gcv;
  unsigned long value_mask = GCForeground | GCBackground;
  XColor screen, exact;

  dispwin = XtWindow (draw);
  gc = XtGetGC (draw, value_mask, &gcv);
  dpy = XtDisplay (top);

  cm = DefaultColormap (dpy, 0);

  if (XAllocNamedColor (dpy, cm, "SteelBlue", &screen, &exact) == 0)
    exit (0);
  bgcol = screen.pixel;
  if (XAllocNamedColor (dpy, cm, "blue", &screen, &exact) == 0)
    exit (0);
  fgcol = screen.pixel;
}


void
createWidgets (void)
{
  mainForm =
    XtVaCreateManagedWidget ("mainForm", xmFormWidgetClass, top, Fix_LTRB,
			     NULL);

  plotForm = XtVaCreateManagedWidget ("plotForm",
				      xmFormWidgetClass, mainForm,
				      Fix_L, Fix_R, Fix_T, NULL);
  commandForm = XtVaCreateManagedWidget ("commandForm",
					 xmFormWidgetClass, mainForm,
					 Fix_L, Fix_R, Fix_B,
					 XmNtopWidget, plotForm,
					 XmNtopAttachment, XmATTACH_WIDGET,
					 NULL);

  draw = XtVaCreateManagedWidget ("draw",
				  xmDrawingAreaWidgetClass, plotForm,
				  XmNheight, 600,
				  XmNwidth, 800, Fix_LTRB, NULL);

  start = XtVaCreateManagedWidget ("START",
				   xmPushButtonWidgetClass, commandForm,
				   Fix_L, Fix_T, Fix_B, NULL);

  stop = XtVaCreateManagedWidget ("STOP",
				  xmPushButtonWidgetClass, commandForm,
				  Fix_T, Fix_B,
				  XmNleftWidget, start,
				  XmNleftAttachment, XmATTACH_WIDGET, NULL);

  save = XtVaCreateManagedWidget ("SAVE",
				  xmPushButtonWidgetClass, commandForm,
				  Fix_T, Fix_B,
				  XmNleftWidget, stop,
				  XmNleftAttachment, XmATTACH_WIDGET, NULL);

  tval = XtVaCreateManagedWidget ("0.0    sec",
				  xmLabelWidgetClass, commandForm,
				  Fix_T, Fix_B,
				  XmNleftWidget, save,
				  XmNleftAttachment, XmATTACH_WIDGET,
				  XmNwidth, 150, NULL);

  quit = XtVaCreateManagedWidget ("EXIT",
				  xmPushButtonWidgetClass, commandForm,
				  Fix_T, Fix_B, Fix_R, NULL);

  XtAddCallback (start, XmNactivateCallback, startCollect, NULL);
  XtAddCallback (stop, XmNactivateCallback, stopCollect, NULL);
  XtAddCallback (save, XmNactivateCallback, saveData, NULL);
  XtAddCallback (quit, XmNactivateCallback, bye, NULL);
  XtAppAddTimeOut (apc, XTIMER, (XtTimerCallbackProc) timer,
		   (XtPointer) NULL);
}


int
main (int argc, char **argv)
{
  openPhoenix ();

  top = XtVaAppInitialize (&apc, "CRO", NULL, 0,	/* command line option list */
			   &argc, argv,	/* command line args */
			   NULL,	/* for missing app-defaults file */
			   NULL);	/* terminate varargs list */

  createWidgets ();
  XtRealizeWidget (top);
  initGraphics ();

  XtAppMainLoop (apc);
  return 0;
}
